/*
 * smalloc.h
 *
 * Contains memory allocator routines
 *
 */

/*
 * The servers "general" memory handler.
 */


#define SMALLOC_MAGIC_ALLOC 0x12FE56A0
#define SMALLOC_MAGIC_FREE  0xCA348E63

/*
 * "safe" malloc. Handles the case when malloc returns NULL.
 * smalloc cannot fail.
 */
extern void *
smalloc(size_t size);


extern void
sfree(void * ptr);	/* it is legal to sfree a NULL pointer */

extern void *
srealloc(void * ptr, size_t size); /* Never fails. It is legal to
				      realloc the NULL ptr. */

/*
 * Allocate temporary memory, which is automatically freed after this
 * atomic call.
 */
void *
tmp_alloc(u_long size);

/*
 * Free all core which is allocated with tmp_alloc(). This is called from
 * end_of_atomic().
 */
void
free_tmp(void);

/*
 * Free all memory used internally in tmp_alloc().
 */
void
free_all_tmp(void);


/*
 * Write statistics of memory usage.
 */

void
dump_smalloc_counts(FILE *stat_file);
